# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'spectramanipulator/dialogs\gui_rename_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(421, 324)
        self.gridLayout_2 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(Dialog)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 2, 0, 1, 1)
        self.leExpression = QtWidgets.QLineEdit(Dialog)
        self.leExpression.setObjectName("leExpression")
        self.gridLayout.addWidget(self.leExpression, 0, 1, 1, 1)
        self.sbOffset = QtWidgets.QSpinBox(Dialog)
        self.sbOffset.setObjectName("sbOffset")
        self.gridLayout.addWidget(self.sbOffset, 1, 1, 1, 1)
        self.leCounterMulFactor = QtWidgets.QLineEdit(Dialog)
        self.leCounterMulFactor.setObjectName("leCounterMulFactor")
        self.gridLayout.addWidget(self.leCounterMulFactor, 2, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.cbTakeNamesFromList = QtWidgets.QCheckBox(Dialog)
        self.cbTakeNamesFromList.setObjectName("cbTakeNamesFromList")
        self.verticalLayout_2.addWidget(self.cbTakeNamesFromList)
        self.leList = QtWidgets.QLineEdit(Dialog)
        self.leList.setObjectName("leList")
        self.verticalLayout_2.addWidget(self.leList)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label.setText(_translate("Dialog", "<html><head/><body><p>Renames selected items. {:2d} - integer counter, {:2f} - float counter, {:2g} - significant digits counter. for more see https://pyformat.info/. For original name slicing, use {start_idx:end_idx}, same from python slicing rules. Eg. expression is \'{:02d}: t = {:} us\', current name is \'167\' and current counter is 16. Resulting name will be \'16: t = 167 us\'. Current counter (integer counter starts with counter offset value) for each item is multiplied by counter mult. factor.</p></body></html>"))
        self.label_2.setText(_translate("Dialog", "Expression:"))
        self.label_3.setText(_translate("Dialog", "Counter offset:"))
        self.label_4.setText(_translate("Dialog", "Counter mult. factor:"))
        self.leCounterMulFactor.setText(_translate("Dialog", "1"))
        self.cbTakeNamesFromList.setText(_translate("Dialog", "Take names from list (separate values by comma):"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())
